@@ -37,7 +37,12 @@
 extern "C" {
 #endif
 
-#ifdef USE_PROJ
+#if PROJ_VERSION_MAJOR >= 6
+#  include <proj.h>
+#if PROJ_VERSION_MAJOR == 6 && PROJ_VERSION_MINOR == 0
+#error "PROJ 6.0 is not supported. Use PROJ 6.1 or later"
+#endif
+#else
 #  include <proj_api.h>
 #if PJ_VERSION >= 470 && PJ_VERSION < 480
    void pj_clear_initcache();
@@ -48,6 +53,7 @@ extern "C" {
 #define wkp_lonlat 1
 #define wkp_gmerc 2
 
+typedef struct projectionContext projectionContext;
 
   typedef struct {
 #ifdef SWIG
@@ -60,13 +66,14 @@ extern "C" {
 #endif
 #ifndef SWIG
     char **args; /* variable number of projection args */
-#ifdef USE_PROJ
+#if PROJ_VERSION_MAJOR >= 6
+    PJ* proj;
+    projectionContext* proj_ctx;
+#else
     projPJ proj; /* a projection structure for the PROJ package */
 #if PJ_VERSION >= 480
     projCtx proj_ctx;
 #endif
-#else
-    void *proj;
 #endif
     geotransformObj gt; /* extra transformation to apply */
 #endif
@@ -75,10 +82,21 @@ extern "C" {
 
 #ifndef SWIG
 
+  typedef struct reprojectionObj reprojectionObj;
+  MS_DLL_EXPORT reprojectionObj* msProjectCreateReprojector(projectionObj* in, projectionObj* out);
+  MS_DLL_EXPORT void msProjectDestroyReprojector(reprojectionObj* reprojector);
+
+  MS_DLL_EXPORT projectionContext* msProjectionContextGetFromPool(void);
+  MS_DLL_EXPORT void msProjectionContextReleaseToPool(projectionContext* ctx);
+  MS_DLL_EXPORT void msProjectionContextPoolCleanup(void);
+
   MS_DLL_EXPORT int msIsAxisInverted(int epsg_code);
-  MS_DLL_EXPORT int msProjectPoint(projectionObj *in, projectionObj *out, pointObj *point);
-  MS_DLL_EXPORT int msProjectShape(projectionObj *in, projectionObj *out, shapeObj *shape);
-  MS_DLL_EXPORT int msProjectLine(projectionObj *in, projectionObj *out, lineObj *line);
+  MS_DLL_EXPORT int msProjectPoint(projectionObj *in, projectionObj *out, pointObj *point); /* legacy interface */
+  MS_DLL_EXPORT int msProjectPointEx(reprojectionObj* reprojector, pointObj *point);
+  MS_DLL_EXPORT int msProjectShape(projectionObj *in, projectionObj *out, shapeObj *shape); /* legacy interface */
+  MS_DLL_EXPORT int msProjectShapeEx(reprojectionObj* reprojector, shapeObj *shape);
+  MS_DLL_EXPORT int msProjectLine(projectionObj *in, projectionObj *out, lineObj *line); /* legacy interface */
+  MS_DLL_EXPORT int msProjectLineEx(reprojectionObj* reprojector, lineObj *line);
   MS_DLL_EXPORT int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect);
   MS_DLL_EXPORT int msProjectionsDiffer(projectionObj *, projectionObj *);
   MS_DLL_EXPORT int msOGCWKT2ProjectionObj( const char *pszWKT, projectionObj *proj, int
@@ -86,7 +104,10 @@ extern "C" {
   MS_DLL_EXPORT char *msProjectionObj2OGCWKT( projectionObj *proj );
 
   MS_DLL_EXPORT void msFreeProjection(projectionObj *p);
+  MS_DLL_EXPORT void msFreeProjectionExceptContext(projectionObj *p);
   MS_DLL_EXPORT int msInitProjection(projectionObj *p);
+  MS_DLL_EXPORT void msProjectionInheritContextFrom(projectionObj *pDst, projectionObj* pSrc);
+  MS_DLL_EXPORT void msProjectionSetContext(projectionObj *p, projectionContext* ctx);
   MS_DLL_EXPORT int msProcessProjection(projectionObj *p);
   MS_DLL_EXPORT int msLoadProjectionString(projectionObj *p, const char *value);
   MS_DLL_EXPORT int msLoadProjectionStringEPSG(projectionObj *p, const char *value);
@@ -101,9 +122,10 @@ extern "C" {
   MS_DLL_EXPORT void msSetPROJ_LIB( const char *, const char * );
   MS_DLL_EXPORT void msProjLibInitFromEnv();
 
-  /* Provides compatiblity with PROJ.4 4.4.2 */
-#ifndef PJ_VERSION
-#  define pj_is_latlong(x)  ((x)->is_latlong)
+  int msProjIsGeographicCRS(projectionObj* proj);
+#if PROJ_VERSION_MAJOR >= 6
+  int msProjectTransformPoints( reprojectionObj* reprojector,
+                                int npoints, double* x, double* y );
 #endif
 
   /*utility functions */
