@@ -1,12 +1,12 @@
 #ifndef _MAPSERVER_CONFIG_H
 #define _MAPSERVER_CONFIG_H
 
-#define USE_PROJ 1
+/* #undef USE_PROJ */
 #define USE_PBF 1
 #define USE_POSTGIS 1
-#define USE_GDAL 1
+/* #undef USE_GDAL */
 /* #undef USE_PIXMAN */
-#define USE_OGR 1
+/* #undef USE_OGR */
 #define USE_WMS_SVR 1
 #define USE_WCS_SVR 1
 #define USE_WFS_SVR 1
@@ -27,7 +27,7 @@
 /* #undef USE_MYSQL */
 /* #undef USE_THREAD */
 #define USE_KML 1
-/* #undef USE_POINT_Z_M */
+#define USE_POINT_Z_M 1
 /* #undef USE_ORACLESPATIAL */
 /* #undef USE_EXEMPI */
 /* #undef USE_XMLMAPFILE */
